#!/usr/bin/env python
# -*- coding: ascii -*-
from __future__ import absolute_import
from __future__ import print_function

import sys, os

if sys.version_info < (3,):
    from future import standard_library
    standard_library.install_aliases()
    from tkSimpleDialog import Dialog
    import tkMessageBox
else:
    # this is only called incorrectly by pylint using python2
    from tkinter.simpledialog import Dialog
    import tkinter.messagebox as tkMessageBox


from tkinter import *

# Import Pillow:
from PIL import Image, ImageTk

from digiplot.fix_distortion import fix_plot_img
from digiplot.plot_area import PlotArea        

class _Dialog(Dialog):
    # use dialogOptions dictionary to set any values in the dialog
    def __init__(self, parent, title = None, dialogOptions=None):
        self.initComplete = 0
        self.dialogOptions = dialogOptions
        Dialog.__init__(self, parent, title)
        
        
class _ReAlign(_Dialog):

    def body(self, body):
        
        body.pack(padx=5, pady=5, fill=BOTH, expand=True) 
        
        dialogframe = Frame(body, width=663, height=627)
        dialogframe.pack(fill=BOTH, expand=YES, side=TOP)
        self.dialogframe = dialogframe

        lbframe = Frame( dialogframe )
        self.Canvas_1_frame = lbframe
        scrollbar = Scrollbar(lbframe, orient=VERTICAL)
        self.Canvas_1 = Canvas(lbframe, width="600", height="600", yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.Canvas_1.yview)
        scrollbar.pack(side=RIGHT, fill=Y)
        self.Canvas_1.pack(side=LEFT, fill=BOTH, expand=YES)
        lbframe.pack(fill=BOTH, expand=YES, side=TOP)

        self.Canvas_1.bind("<ButtonRelease-1>", self.Canvas_1_Click)

        
        # Mouse Wheel for Zooming in and out
        self.Canvas_1.bind("<MouseWheel>",self.MouseWheelHandler) # Windows Binding
        self.Canvas_1.bind("<Button-4>",self.MouseWheelHandler) # Linux Binding
        self.Canvas_1.bind("<Button-5>",self.MouseWheelHandler) # Linux Binding   

        self.Canvas_1.bind('<Up>', self.MakeBigger)
        self.Canvas_1.bind('<Down>', self.MakeSmaller)
        
        #body.unbind_all('<Escape>')
        self.Canvas_1.bind_all('<Key-1>', self.Key_Actions)
        self.Canvas_1.bind_all('<Key-2>', self.Key_Actions)
        self.Canvas_1.bind_all('<Key-3>', self.Key_Actions)
        self.Canvas_1.bind_all('<Key-4>', self.Key_Actions)

        
        self.Canvas_1.bind("<Button-3>", self.Canvas_Begin_End_Drag)
        self.Canvas_1.bind("<B3-Motion>", self.Canvas_Drag_Axes)
        self.Canvas_1.bind("<Motion>", self.Canvas_Hover)
        self.Canvas_1.bind("<Enter>", self.Canvas_Enter)
        self.Canvas_1.bind("<Leave>", self.Canvas_Leave)
        
        self.Canvas_1.bind("<Configure>", self.Canvas_1_Resize)
        
        self.resizable(1,1) # Linux may not respect this
        # >>>>>>insert any user code below this comment for section "top_of_init"
        
        self.Canvas_1.focus_set()

        self.PA = PlotArea(w_canv=600, h_canv=600) # will hold PlotArea() when it is opened
        
        # use a copy of submitted image
        self.PA.set_img( self.dialogOptions['img'].copy() )
        
        self.is_dragging = False
        self.last_right_click_pos = (0,0) # any action calling Canvas_Find_Closest will set

        self.last_hover_pos = (0,0)
        self.in_canvas = False

        self.canvas_tooltip_num = 0
        self.canvas_tooltip_inc = 1 # set > 1 to skip some options
        self.canvas_tooltip_strL = ['Left Click Upper Left.', 'Left Click Upper Right',
                                    'Left Click Lower Right.', 'Left Click Lower Left']
        # Units NEED to be fraction of img
        self.canvas_click_posL = [None, None, None, None] # only useful if all None's replaced
        
        if self.dialogOptions.get('use_3_point',False):
            self.canvas_click_posL[1] = (1000, 1000)
        
        self.is_first_display = True
        self.all_done = False
        

    def Key_Actions(self, event):
        #print('event.char =',event.char)
        if event.char=='1':
            self.PA.zoom_to_quadrant(qname='UL')
        elif event.char=='2':
            self.PA.zoom_to_quadrant(qname='UR')
        elif event.char=='3':
            self.PA.zoom_to_quadrant(qname='LR')
        elif event.char=='4':
            self.PA.zoom_to_quadrant(qname='LL')
            
        self.fill_canvas()

    def MakeBigger(self, event):
        #print('MakeBigger')
        self.PA.zoom_in(zoom_factor=0.1)
        self.fill_canvas()
        
    def MakeSmaller(self, event):
        #print('MakeSmaller')
        self.PA.zoom_out(zoom_factor=0.1)
        self.fill_canvas()

    def fill_canvas(self):
        if self.all_done:
            return
        
        if self.is_first_display:
            self.PA.zoom_to_quadrant(qname='UL')
            self.is_first_display = False
        
        # make coordinate axes
        try:
            self.Canvas_1.delete("all")
        except:
            pass
        
        # Place click directions onto canvas
        if self.canvas_tooltip_num < len(self.canvas_tooltip_strL):
            tt_str = self.canvas_tooltip_strL[ self.canvas_tooltip_num ]
        else:
            tt_str = ''
        
        self.photo_image = self.PA.get_tk_photoimage(greyscale=True, text=tt_str)
        self.Canvas_1.create_image(0,0, anchor=NW, image=self.photo_image )
        

        # Show cross-hairs for picking axis points
        if self.in_canvas:
            x,y = self.last_hover_pos
            self.Canvas_1.create_line(x, 0, x, self.PA.h_canv, fill="magenta", width=3, dash=(5,) )
            self.Canvas_1.create_line(0, y, self.PA.w_canv, y, fill="magenta", width=3, dash=(5,) )

        

    def Canvas_1_Resize(self, event):
        #w_canv = int( self.Canvas_1.winfo_width() )
        #h_canv = int( self.Canvas_1.winfo_height() )
        #if w_canv>0 and h_canv>0:
            
        self.PA.set_canvas_wh(event.width, event.height)
        self.fill_canvas()

    def MouseWheelHandler(self, event):
        #print('MouseWheelHandler event.num =', event.num)

        if event.num == 5 or event.delta < 0:
            result = -1 
            #self.PA.zoom_in(zoom_factor=0.1)
            self.PA.zoom_into_ij(event.x, event.y, zoom_factor=0.1)
        else:
            result = 1 
            #self.PA.zoom_out(zoom_factor=0.1)
            self.PA.zoom_out_from_ij(event.x, event.y, zoom_factor=0.1)
            
        self.fill_canvas()

    def Canvas_Begin_End_Drag(self, event):
        
        self.is_dragging = not self.is_dragging
        ix = int(event.x)
        iy = int(event.y)
        
        self.last_right_click_pos = (ix, iy)
        

    def Canvas_Drag_Axes(self, event):
        
        di = self.last_right_click_pos[0] - event.x
        dj = self.last_right_click_pos[1] - event.y
        self.PA.adjust_offset(di, dj)
        
        self.last_right_click_pos = (event.x, event.y)
        
        self.fill_canvas()

    def Canvas_Enter(self, event):
        self.in_canvas = True
        self.fill_canvas()
                
        
    def Canvas_Leave(self, event):
        self.in_canvas = False
        self.fill_canvas()


    def Canvas_Hover(self, event):
            
        self.last_hover_pos = (event.x, event.y)
        self.fill_canvas()
            

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "compID=2"
    def Canvas_1_Click(self, event): #click method for component ID=2
        pass
        # >>>>>>insert any user code below this comment for section "compID=2"
        # replace, delete, or comment-out the following
        #print("executed method Canvas_1_Click")
        #print("clicked in canvas at x,y =",event.x,event.y)
        #w = int(self.Canvas_1.cget("width"))
        #h = int(self.Canvas_1.cget("height"))
        
        w = int(self.Canvas_1.winfo_width())
        h = int(self.Canvas_1.winfo_height())

        if self.canvas_tooltip_num < len(self.canvas_tooltip_strL):
            tt_str = self.canvas_tooltip_strL[ self.canvas_tooltip_num ]
            
            # For Now, units are fi,fj
            fi, fj = self.PA.get_fifj_at_ij(event.x, event.y)
            self.canvas_click_posL[self.canvas_tooltip_num] = \
                        (self.PA.get_img_i_from_img_fi(fi),  \
                         self.PA.get_img_j_from_img_fj(fj) )
            
            if self.canvas_tooltip_num==0:
                self.title("Upper Left = " + str(self.canvas_click_posL[self.canvas_tooltip_num]))
                self.canvas_tooltip_num += self.canvas_tooltip_inc
                
                if self.dialogOptions.get('use_3_point',False):
                    self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==1:
                self.title("Upper Right = " + str(self.canvas_click_posL[self.canvas_tooltip_num]))
                self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==2:
                self.title("Lower Right = " + str(self.canvas_click_posL[self.canvas_tooltip_num]))
                self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==3:
                self.title("Lower Left = " + str(self.canvas_click_posL[self.canvas_tooltip_num]))
                self.canvas_tooltip_num += self.canvas_tooltip_inc
                    
                
            
            self.canvas_tooltip_inc = 1 # Just in case it is >1 for single options

        
            if self.canvas_tooltip_num==0:
                self.PA.zoom_to_quadrant(qname='UL')
            elif self.canvas_tooltip_num==1:
                self.PA.zoom_to_quadrant(qname='UR')
            elif self.canvas_tooltip_num==2:
                self.PA.zoom_to_quadrant(qname='LR')
            elif self.canvas_tooltip_num==3:
                self.PA.zoom_to_quadrant(qname='LL')
            else:
                self.PA.fit_img_on_canvas()


            if not None in self.canvas_click_posL:
                UL, UR, LR, LL = self.canvas_click_posL
                
                # May need to assume that UR is assumed to be a rotation
                if self.dialogOptions.get('use_3_point',False):
                    di = LR[0] - LL[0]
                    dj = LR[1] - LL[1]
                    UR = (UL[0]+di, UL[1]+dj)
                
                #print('UL, UR, LR, LL =',UL, UR, LR, LL)
                self.img_fixed = fix_plot_img( UL, UR, LR, LL, self.PA.img)
                self.PA.set_img( self.img_fixed )
                
                self.ShowInfo( title='Image Is Realigned', message='''Use Cross-Hairs to examine orthogonality
                If satisfied, hit "OK" button on main screen.''')
                
            
            self.fill_canvas()
                

    # standard message dialogs... showinfo, showwarning, showerror
    def ShowInfo(self, title='Title', message='your message here.'):
        tkMessageBox.showinfo( title, message )
        return
        

    # tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "dialog_validate"
    def validate(self):
        self.result = {} # return a dictionary of results
        
        #self.Canvas_1.unbind_all('<Key>')
        self.all_done = True
        
        self.result["img_fixed"] = self.img_fixed.copy()
        return 1
# tk_happy generated code. DO NOT EDIT THE FOLLOWING. section "end"


    def apply(self):
        #print('apply called')
        pass

class _Testdialog:
    def __init__(self, master):
        frame = Frame(master, width=300, height=300)
        frame.pack()
        self.master = master
        self.x, self.y, self.w, self.h = -1,-1,-1,-1
        
        self.Button_1 = Button(text="Test Dialog", relief="raised", width="15")
        self.Button_1.place(x=84, y=36)
        self.Button_1.bind("<ButtonRelease-1>", self.Button_1_Click)

    def Button_1_Click(self, event): #click method for component ID=1

        # Load the original image:
        #img = Image.open("rot_poly_m3.jpg")
        img = Image.open("Tank_ExpEff_3Point.jpg")
        dialogOptions = {'img':img, 'use_3_point':True}
        
        dialog = _ReAlign(self.master, "Test Dialog",dialogOptions=dialogOptions)
        print('===============Result from Dialog====================')
        print(dialog.result)
        print('=====================================================')

def main():
    root = Tk()
    app = _Testdialog(root)
    root.mainloop()

if __name__ == '__main__':
    main()



from math import sqrt, cos, sin, degrees, asin, atan
# Import Pillow:
from PIL import Image

# Load the original image:
for i_img, img_name in enumerate(["rot_poly_p3.jpg", "rot_poly_m3.jpg"]):
    img = Image.open(img_name)
    
    if i_img==0:
        #img_name = "rot_poly_p3.jpg"
        xmin = (135, 666)
        xmax = (877, 627)
    else:
        #img_name = "rot_poly_m3.jpg"
        xmin = (106, 627)
        xmax = (848, 666)

    print( img_name )
    dy = float( xmax[1] - xmin[1] )
    dx = float( xmax[0] - xmin[0] )
    print('dx=%g, dy=%g'%(dx,dy))
    ang = atan( dy/dx )
    print('ang=%g(%g)'%(ang, degrees(ang)))
        
    print('='*55)
    
    img_name_fixed = 'fixed_' + img_name
    img_fix = img.rotate(degrees(ang), resample=Image.BICUBIC)
    img_fix.save(img_name_fixed)





#!/usr/bin/env python
# -*- coding: ascii -*-
from __future__ import absolute_import
from __future__ import print_function

import math
import sys
from PIL import Image, ImageTk, ImageFont, ImageDraw

try:
    from PIL import ImageGrab
    HAS_IMAGEGRAB = True
except:
    HAS_IMAGEGRAB = False

if sys.version_info < (3,):
    from cStringIO import StringIO
else:
    from io import BytesIO as StringIO
import os
from digiplot.sample_img import TEST_IMAGE

here = os.path.abspath(os.path.dirname(__file__))
font_path = os.path.join( here, 'fonts', 'FreeSerifBoldItalic.ttf' )

freefont10  =  ImageFont.truetype ( font_path, 10 )
freefont16  =  ImageFont.truetype ( font_path, 16 )
freefont24  =  ImageFont.truetype ( font_path, 24 )
freefont36  =  ImageFont.truetype ( font_path, 36 )
freefont48  =  ImageFont.truetype ( font_path, 48 )
freefont72  =  ImageFont.truetype ( font_path, 72 )


def clamp(n, minn, maxn):
    if n < minn:
        return minn
    elif n > maxn:
        return maxn
    else:
        return n

class PlotArea(object):
    """
    Logic for interpreting a plot image on a display canvas.
    The image can be zoomed and panned within the canvas area.
    The image is ALWAYS displayed at UL corner (0,0), but can be at many zoom/pan levels.
    
    fi,fj are the fractional i,j locations within the original image. (value = 0.0 to 1.0)
    """
    
    def __init__(self, w_canv=1000, h_canv=1000):
        self.w_canv = w_canv
        self.h_canv = h_canv
        
        test_img_data = TEST_IMAGE
        self.img =  Image.open(StringIO(test_img_data))
        self.w_img, self.h_img = self.img.size

        self.calc_nominal_zoom()

        self.fi_origin = 0.0
        self.fj_origin = 1.0 # lower left
        self.fimax = 1.0
        self.fjmax = 0.0
        
        self.fi_offset = 0.0 # when upper left of canvas is not upper left of image
        self.fj_offset = 0.0
        
        self.x_origin = 0.0 # units on the image plot
        self.y_origin = 0.0 # units on the image plot
        self.xmax = 10.0 # units on the image plot
        self.ymax = 10.0 # units on the image plot
        
        self.log_x = False
        self.log_y = False
        
        self.log10_x_origin = 1.0 # units on the image plot
        self.log10_y_origin = 1.0 # units on the image plot
        self.log10_xmax = 2.0 # units on the image plot
        self.log10_ymax = 2.0 # units on the image plot
        
    def set_log_x(self):
        self.log_x = True
        if self.x_origin <= 0.0:
            self.x_origin = 0.1
        self.log10_x_origin = math.log10( self.x_origin )
        
    def set_linear_x(self):
        self.log_x = False
        
    def set_log_y(self):
        self.log_y = True
        if self.y_origin <= 0.0:
            self.y_origin = 0.1
        self.log10_y_origin = math.log10( self.y_origin )
        
    def set_linear_y(self):
        self.log_y = False


    def set_fraction_offset(self, fi=0.0, fj=0.0):
        
        # calc max values to limit setting
        i_img_shown = min(self.w_img, self.w_canv /  self.img_zoom)
        j_img_shown = min(self.h_img, self.h_canv /  self.img_zoom)
        
        fi_off_max = clamp( 1.0 - float(i_img_shown)/float(self.w_img), 0., 1.)
        fj_off_max = clamp(1.0 - float(j_img_shown)/float(self.h_img), 0., 1.)
        #print('fi_off_max=%g, fj_off_max=%g, fi=%g, fj=%g'%(fi_off_max, fj_off_max, fi, fj))
        
        self.fi_offset = clamp(fi, 0., fi_off_max) # when upper left of canvas is not upper left of image
        self.fj_offset = clamp(fj, 0., fj_off_max)

    def set_canvas_wh(self, w_canv, h_canv):
        self.w_canv = w_canv
        self.h_canv = h_canv
        self.calc_nominal_zoom()

    def set_zoom(self, new_zoom):
        self.img_zoom = max(new_zoom, self.fit_img_zoom)

    def zoom_in(self, zoom_factor=0.1):
        self.set_zoom( self.img_zoom * (1.0 + zoom_factor) )

    def zoom_out(self, zoom_factor=0.1):
        self.set_zoom( self.img_zoom / (1.0 + zoom_factor))
        
    def zoom_to_quadrant(self, qname='UL'):
        
        x_zoom = float(self.w_canv) / float(self.w_img)
        y_zoom = float(self.h_canv) / float(self.h_img)
        self.img_zoom =  max(x_zoom, y_zoom) * 1.5
        
        qname = qname.upper()
        if qname=='UL':
            self.set_fraction_offset(fi=0.0, fj=0.0)
        elif qname=='UR':
            self.set_fraction_offset(fi=1.0, fj=0.0)
        elif qname=='LR':
            self.set_fraction_offset(fi=1.0, fj=1.0)
        elif qname=='LL':
            self.set_fraction_offset(fi=0.0, fj=1.0)
        else:
            self.fit_img_on_canvas()

    def zoom_into_ij(self,i,j, zoom_factor=0.1):
        fi = self.get_img_fi_from_canvas_i( i )
        fj = self.get_img_fj_from_canvas_j( j )
        
        #fi_max = self.get_img_fi_from_canvas_i( self.w_canv )
        #fj_max = self.get_img_fj_from_canvas_j( self.h_canv )
        #print('fi_max=%g,  fj_max=%g'%(fi_max, fj_max))

        # Want new fi, fj same as old values but with new zoom factor
        self.set_zoom( self.img_zoom * (1.0 + zoom_factor) )
        
        fi_new = self.get_img_fi_from_canvas_i( i )
        fj_new = self.get_img_fj_from_canvas_j( j )
        
        fi_final = self.fi_offset + fi - fi_new
        fj_final = self.fj_offset + fj - fj_new
        
        self.set_fraction_offset( fi=fi_final, fj=fj_final )
        
    def zoom_out_from_ij(self,i,j, zoom_factor=0.1):
        fi = self.get_img_fi_from_canvas_i( i )
        fj = self.get_img_fj_from_canvas_j( j )

        # Want new fi, fj same as old values but with new zoom factor
        self.set_zoom( self.img_zoom / (1.0 + zoom_factor))
        
        fi_new = self.get_img_fi_from_canvas_i( i )
        fj_new = self.get_img_fj_from_canvas_j( j )
        
        #self.fi_offset += fi - fi_new
        #self.fj_offset += fj - fj_new
        self.set_fraction_offset( fi=self.fi_offset + fi - fi_new, fj=self.fj_offset + fj - fj_new )

    def fit_img_on_canvas(self):
        self.calc_nominal_zoom()
        self.fi_offset = 0.0 # when upper left of canvas is not upper left of image
        self.fj_offset = 0.0


    def calc_nominal_zoom(self):
        x_zoom = float(self.w_canv) / float(self.w_img)
        y_zoom = float(self.h_canv) / float(self.h_img)
        self.img_zoom = min(x_zoom, y_zoom)
        
        self.fit_img_zoom = self.img_zoom

    def open_img_file(self, img_path):
        try:
            img =  Image.open(img_path)
            w_img, h_img = img.size
            print('Opened image file:',img_path,'  size=',img.size)
            self.set_img( img )
            return True
        except:
            print('==========> Error opening image file:',img_path)
            return False
    
    def set_img_from_clipboard(self):
        if not HAS_IMAGEGRAB:
            return False
        
        
        img = ImageGrab.grabclipboard()
        if isinstance(img, Image.Image):
            self.set_img( img )
            return True
        else:
            print('WARNING... Pasting Clipboard Image Failed.')
            return False
    
    def set_img(self, img):
        
        self.img = img
        self.w_img, self.h_img = img.size
        
        self.calc_nominal_zoom()

        self.fi_origin = 0.0
        self.fj_origin = 1.0 # lower left
        self.fimax = 1.0
        self.fjmax = 0.0
        
        self.fi_offset = 0.0 # when upper left of canvas is not upper left of image
        self.fj_offset = 0.0
        
        self.x_origin = 0.0 # units on the image plot
        self.y_origin = 0.0 # units on the image plot
        self.xmax = 10.0 # units on the image plot
        self.ymax = 10.0 # units on the image plot
        
        self.log_x = False
        self.log_y = False
        
        self.log10_x_origin = 1.0 # units on the image plot
        self.log10_y_origin = 1.0 # units on the image plot
        self.log10_xmax = 2.0 # units on the image plot
        self.log10_ymax = 2.0 # units on the image plot
        
    
    def get_zoomed_offset_img(self, greyscale=False, text='', show_linlog_text=False):
        
        fi_min = max(0.0, self.get_img_fi_from_canvas_i( 0 ))
        fi_max = min(1.0, self.get_img_fi_from_canvas_i( self.w_canv ))
        fj_min = max(0.0, self.get_img_fj_from_canvas_j( 0 ))
        fj_max = min(1.0, self.get_img_fj_from_canvas_j( self.h_canv ))
        
        imin = int( self.w_img * fi_min )
        imax = int( self.w_img * fi_max )
        jmin = int( self.h_img * fj_min )
        jmax = int( self.h_img * fj_max )
        
        img_slice = self.img.crop( (imin,jmin, imax,jmax) )
        wz = int((imax-imin+1)*self.img_zoom)
        hz = int((jmax-jmin+1)*self.img_zoom)
        img_slice_resized = img_slice.resize( (wz, hz), Image.ANTIALIAS)
        
        if (wz>self.w_canv) or (hz>self.h_canv):
            bbox = (0,0, min(wz,self.w_canv), min(hz,self.h_canv))
            img_slice_resized = img_slice_resized.crop( bbox )
            #print('... bbox resized to:', bbox)
            
        if greyscale:
            img_slice_resized =  img_slice_resized.convert('LA')
            
        # place text onto plot
        if text or show_linlog_text:
            # Make RGBA image to put text on
            img_slice_resized =  img_slice_resized.convert('RGBA')
            w,h = img_slice_resized.size
            
            d = max(w,h)
            img_square = Image.new('RGBA', (d,d))
            draw = ImageDraw.Draw(img_square)
            
            def get_font_for_size( s, w_lim ):
                myfont = freefont10
                for test_font in [freefont16, freefont24, freefont36, freefont48, freefont72]:
                    wtext, htext = test_font.getsize(s)
                    if wtext >= w_lim:
                        return myfont
                    myfont = test_font
                return myfont

            # yaxis text
            if show_linlog_text:
                if self.log_y:
                    ylab = 'log scale'
                    color = (255,0,0,120) # red
                else:
                    ylab = 'linear scale'
                    color = (0,255,0,120) # green
                myfont = get_font_for_size( 'logear scale', min(w,h)/2 )
                di, dj = myfont.getsize(ylab)
                itxt = (h-di)/2
                draw.text( (itxt,d-dj),ylab, font=myfont, fill=color )
            
            # rotate image
            img_temp = img_square.rotate(270)
            img_new = img_temp.crop( (0,0, w,h) )
            draw = ImageDraw.Draw(img_new)

            # xaxis text
            if show_linlog_text:
                if self.log_x:
                    xlab = 'log scale'
                    color = (255,0,0,120) # red
                else:
                    xlab = 'linear scale'
                    color = (0,255,0,120) # green
                di, dj = myfont.getsize(xlab)
                itxt = (w-di)/2
                draw.text( (itxt,h-dj),xlab, font=myfont, fill=color )

            # center text (if any)
            if text:
                myfont = get_font_for_size( text, w )
                di, dj = myfont.getsize(text)
                itxt = (w-di)/2
                jtxt = (h-dj)/2
                
                draw.text( (itxt, jtxt), text, font=myfont, fill=(255,0,255,120) ) # magenta

                
            #img_slice_resized = Image.blend(img_slice_resized, img_new, 0.5) 
            img_slice_resized = Image.alpha_composite(img_slice_resized, img_new)

        return img_slice_resized
    
    def get_tk_photoimage(self, greyscale=False, text='', show_linlog_text=False):
        img_slice_resized = self.get_zoomed_offset_img(greyscale=greyscale, text=text, 
                                                       show_linlog_text=show_linlog_text)
        return ImageTk.PhotoImage( img_slice_resized )
    
        
    #def get_img_fi_from_canvas_i(self, i):
    #    i_zoom = self.w_img * self.fi_offset * self.img_zoom + i
    #    i_zoom_max = self.w_img * self.img_zoom
    #    return float(i_zoom) / float(i_zoom_max)

    def get_img_i_from_img_fi(self, fi):
        return self.w_img * fi
    def get_img_j_from_img_fj(self, fj):
        return self.h_img * fj


    def get_canvas_i_from_img_fi(self, fi):
        i_off_screen = self.w_img * self.fi_offset
        i_total =  self.w_img * fi
        return (i_total - i_off_screen) * self.img_zoom

    def get_canvas_j_from_img_fj(self, fj):
        j_off_screen = self.h_img * self.fj_offset
        j_total =  self.h_img * fj
        return (j_total - j_off_screen) * self.img_zoom

    def get_img_fi_from_canvas_i(self, i):
        """Given i on canvas, get fraction of img width that i corresponds to."""
        i_off_screen = self.w_img * self.fi_offset
        i_zoom =  float(i) / self.img_zoom
        #i_zoom_max = self.w_img * self.img_zoom
        return float(i_off_screen+i_zoom) / float(self.w_img)
        
    def get_img_fj_from_canvas_j(self, j):
        """Given j on canvas, get fraction of img height that j corresponds to."""
        j_off_screen = self.h_img * self.fj_offset
        j_zoom = float(j) / self.img_zoom
        #j_zoom_max = self.h_img * self.img_zoom
        return float(j_off_screen+j_zoom) / float(self.h_img)

    def adjust_offset(self, di, dj):
        dfi = float(di) / self.img_zoom / float(self.w_img)
        dfj = float(dj) / self.img_zoom / float(self.h_img)
        
        #self.fi_offset = clamp( dfi+self.fi_offset, 0., 1.)
        #self.fj_offset = clamp( dfj+self.fj_offset, 0., 1.)
        self.set_fraction_offset( fi=self.fi_offset + dfi, fj=self.fj_offset + dfj )
        

    def define_origin_ij(self, i, j):
        """
        Place the origin of the 2D plot at image coordinates i,j.
        i and j are in pixel coordinates of the visible/zoomed portion of the canvas.
        """
        self.fi_origin = self.get_img_fi_from_canvas_i( i )
        self.fj_origin = self.get_img_fj_from_canvas_j( j )
        
    def set_origin_xy(self, x, y):
        self.set_x_origin(x)
        self.set_y_origin(y)
        
    def set_x_origin(self, x):
        self.x_origin = x
        if self.log_x and self.x_origin<=0.0:
            self.x_origin = 0.1
        if self.log_x:
            self.log10_x_origin = math.log10( self.x_origin )
    
    def set_y_origin(self, y):
        self.y_origin = y
        if self.log_y and self.y_origin<=0.0:
            self.y_origin = 0.1
        if self.log_y:
            self.log10_y_origin = math.log10( self.y_origin )
    
    def set_x_max(self, x):
        self.xmax = x
        if self.log_x and self.xmax<=0.0:
            self.xmax = 10.0
        if self.log_x:
            self.log10_xmax = math.log10( self.xmax )
    
    def set_y_max(self, y):
        self.ymax = y
        if self.log_y and self.ymax<=0.0:
            self.ymax = 10.0
        if self.log_y:
            self.log10_ymax = math.log10( self.ymax )
    
    def set_ix_origin(self, i, x):
        self.fi_origin = self.get_img_fi_from_canvas_i( i )
        self.set_x_origin(x)
    
    def set_jy_origin(self, j, y):
        self.fj_origin = self.get_img_fj_from_canvas_j( j )
        self.set_y_origin(y)
    
    def set_imax_xmax(self, imax, xmax):
        self.fimax = self.get_img_fi_from_canvas_i( imax )
        self.set_x_max( xmax )
    
    def set_jmax_ymax(self, jmax, ymax):
        self.fjmax = self.get_img_fj_from_canvas_j( jmax )
        self.set_y_max( ymax )
    
    def get_xy_at_fifj(self, fi, fj):
    
        di = fi - self.fi_origin
        dj = self.fj_origin - fj # note LL vs UL
    
        if self.log_x:
            dx = self.log10_xmax - self.log10_x_origin
        else:
            dx = self.xmax - self.x_origin
        
        if self.log_y:
            dy = self.log10_ymax - self.log10_y_origin
        else:
            dy = self.ymax - self.y_origin
    
        try:
            if self.log_x:
                x = 10.0**( self.log10_x_origin + dx * di / (self.fimax - self.fi_origin) )
            else:
                x = self.x_origin + dx * di / (self.fimax - self.fi_origin)
            
            if self.log_y:
                y = 10.0**( self.log10_y_origin + dy * dj / (self.fj_origin - self.fjmax) ) # note LL vs UL
            else:
                y = self.y_origin + dy * dj / (self.fj_origin - self.fjmax) # note LL vs UL
                
        except:
            return None, None
    
        return x,y
    
    def get_fifj_at_ij(self, i,j):
        fi = self.get_img_fi_from_canvas_i( i )
        fj = self.get_img_fj_from_canvas_j( j )
        return fi, fj
    
    def get_xy_at_ij(self, i,j):
        fi = self.get_img_fi_from_canvas_i( i )
        fj = self.get_img_fj_from_canvas_j( j )
        return self.get_xy_at_fifj(fi, fj)


    def get_canvas_i(self, x_float):
        if self.log_x:
            x10 = math.log10( x_float )
            fx = (x10-self.log10_x_origin) / (self.log10_xmax-self.log10_x_origin)
        else:
            fx = (x_float-self.x_origin) / (self.xmax-self.x_origin)
        
        f_plot = self.fimax - self.fi_origin # fraction of canvas holding plot
        i_plot = fx * f_plot * self.w_img * self.img_zoom # i value into plot from origin
        i_offset = self.fi_offset * self.w_img * self.img_zoom
        i_origin = self.fi_origin * self.w_img * self.img_zoom
        
        i = int( i_plot - i_offset + i_origin )
        
        if i>=0 and i<=self.w_canv:
            return i
        else:
            return -1 # if not on canvas
        
    def get_canvas_j(self, y_float):
        if self.log_y:
            y10 = math.log10( y_float )
            fy = (y10-self.log10_y_origin) / (self.log10_ymax-self.log10_y_origin)
        else:
            fy = (y_float-self.y_origin) / (self.ymax-self.y_origin)
        
        f_plot = self.fj_origin - self.fjmax # fraction of canvas holding plot
        j_plot = fy * f_plot * self.h_img * self.img_zoom # i value into plot from origin
        
        j_offset = self.fj_offset * self.h_img * self.img_zoom
        j_origin = self.fj_origin * self.h_img * self.img_zoom
        
        j = int( j_origin - j_offset - j_plot  )
        if j>=0 and j<=self.h_canv:
            return j
        else:
            return -1 # if not on canvas

    def get_ij_at_xy(self, x,y):
        
        i = self.get_canvas_i( x )
        j = self.get_canvas_j( y )
        if i>=0 and j>=0:
            return i,j
        else:
            return -1, -1 # if not on canvas
            
    def x_is_visible(self, x):
        return self.get_canvas_i(x) >= 0
            
    def y_is_visible(self, y):
        return self.get_canvas_j(y) >= 0

if __name__ == '__main__':
    PA = PlotArea(w_canv=600, h_canv=600)
    #PA.open_img_file( 'Cu_cte.png' )
    #PA.set_fraction_offset(fi=0.2, fj=0.3)
    #PA.zoom_out(zoom_factor=0.5)
    
    img_slice_resized = PA.get_zoomed_offset_img(greyscale=True, show_linlog_text=True)
    print('img_slice_resized.size=',img_slice_resized.size)
    img_slice_resized.save('test_img.png')
    
    PA.set_ix_origin(76, 0.0)
    PA.set_jy_origin(432, 0.0)
    
    PA.set_imax_xmax(540, 20.0)
    PA.set_jmax_ymax(48, 120.0)
    
    if 0:
        for x,i_exp in [(0.,76), (5.,190), (10.,306), (15.,424), (20.,540)]:
            print( 'x=%g,  i=%i, i_exp=%i'%(x, PA.get_canvas_i(x),i_exp))

        print('='*55)
        for y,j_exp in [(0.,432), (20.,369), (40.,304), (80.,177), (120.,49)]:
            print( 'y=%g,  j=%i, j_exp=%i'%(y, PA.get_canvas_j(y),j_exp))
            
        print()
        print( 'x,y at 306,304 =',PA.get_xy_at_ij(306,304) )
    else:
        print('='*20,' Offset')
        PA.set_fraction_offset(fi=0.1, fj=0.1)
        
        for x,i_exp in [(0.,16), (5.,131), (10.,248), (15.,365), (20.,482)]:
            print( 'x=%g,  i=%i, i_exp=%i'%(x, PA.get_canvas_i(x),i_exp))

        print('='*55)
        for y,j_exp in [(0.,383), (20.,322), (40.,257), (80.,130), (120.,0)]:
            print( 'y=%g,  j=%i, j_exp=%i'%(y, PA.get_canvas_j(y),j_exp))
            
        print()
        print( 'x,y at 481,0 =',PA.get_xy_at_ij(481,0) )

        img_slice_resized = PA.get_zoomed_offset_img(greyscale=True, show_linlog_text=True)
        img_slice_resized.save('test_offset_img.png')



import unittest
# import unittest2 as unittest # for versions of python < 2.7

"""
        Method                  Checks that
self.assertEqual(a, b)           a == b   
self.assertNotEqual(a, b)        a != b   
self.assertTrue(x)               bool(x) is True  
self.assertFalse(x)              bool(x) is False     
self.assertIs(a, b)              a is b
self.assertIsNot(a, b)           a is not b
self.assertIsNone(x)             x is None 
self.assertIsNotNone(x)          x is not None 
self.assertIn(a, b)              a in b
self.assertNotIn(a, b)           a not in b
self.assertIsInstance(a, b)      isinstance(a, b)  
self.assertNotIsInstance(a, b)   not isinstance(a, b)  

See:
      https://docs.python.org/2/library/unittest.html
         or
      https://docs.python.org/dev/library/unittest.html
for more assert options
"""

import sys, os

here = os.path.abspath(os.path.dirname(__file__)) # Needed for py.test
up_one = os.path.split( here )[0]  # Needed to find digiplot development version
if here not in sys.path[:2]:
    sys.path.insert(0, here)
if up_one not in sys.path[:2]:
    sys.path.insert(0, up_one)

from digiplot.plot_area import PlotArea

class MyTest(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.myclass = PlotArea(w_canv=1000, h_canv=1000)
        self.myclass.open_img_file( os.path.join(here,'Cu_cte.png') )

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        del( self.myclass )

    def test_should_always_pass_cleanly(self):
        """Should always pass cleanly."""
        pass

    def test_image_size(self):
        C = self.myclass
        self.assertEqual( (792, 612), (C.w_img,C.h_img) )

    def test_myclass_existence(self):
        C = self.myclass
        self.assertTrue(C)
        self.assertEqual(C.fi_origin, 0.0)
        self.assertEqual(C.fj_origin, 1.0)

    def test_set_origin_xy(self):
        C = self.myclass
        C.set_origin_xy( 10., 1000. )
        self.assertEqual( (C.x_origin,C.y_origin), (10., 1000.))


if __name__ == '__main__':
    # Can test just this file from command prompt
    #  or it can be part of test discovery from nose, unittest, pytest, etc.
    unittest.main()




#!/usr/bin/env python
# -*- coding: ascii -*-
from __future__ import absolute_import
from __future__ import print_function

r"""
A tkinter code that helps users digitize curves from plot images.

If a 2D plot is available only on paper or an image file, but the user needs the x,y data pairs in digital form, DigiPlot will help to create those x,y data pairs.


DigiPlot
Copyright (C) 2017  Charlie Taylor

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

-----------------------

"""
import os
here = os.path.abspath(os.path.dirname(__file__))
import sys
if sys.version_info < (3,):
    from future import standard_library
    standard_library.install_aliases()
    import tkFileDialog
    import tkMessageBox
    import tkSimpleDialog
else:
    import tkinter.filedialog as tkFileDialog
    import tkinter.messagebox as tkMessageBox
    import tkinter.simpledialog as tkSimpleDialog

#  Need to redefine object here
# pylint: disable=W0622, W0703, W0612, W0403, C0330
from builtins import object

from tkinter import Menu, StringVar, Label, SUNKEN, SW, X, BOTTOM, Frame, NE, NW,\
    BOTH, TOP, Button, LEFT, SE, Scrollbar, VERTICAL, Text, RIGHT, Y, END, Tk,\
    Canvas, Listbox, Entry, N, S, E, W, YES, Toplevel, ALL, Checkbutton

from PIL import Image, ImageTk
from digiplot.plot_area import PlotArea, HAS_IMAGEGRAB
from digiplot.realign_Dialog import _ReAlign

# for multi-file projects see LICENSE file for authorship info
# for single file projects, insert following information
__author__ = 'Charlie Taylor'
__copyright__ = 'Copyright (c) 2016 Charlie Taylor'
__license__ = 'GPL-3'
exec( open(os.path.join( here,'_version.py' )).read() )  # creates local __version__ variable
__email__ = "cet@appliedpython.com"
__status__ = "3 - Alpha" # "3 - Alpha", "4 - Beta", "5 - Production/Stable"


class Point( object ):
    def __init__(self, x, y):
        
        self.x = x
        self.y = y
        
    def get_str(self):
        return '%g, %g'%(self.x, self.y)
    
    def set_xy(self, x, y):
        self.x = x
        self.y = y
    
    def get_xy(self):
        return self.x, self.y


class DigiPlot(object):
    """A tkinter code that helps users digitize curves from plot images."""

    def __init__(self, master):
        
        self.initComplete = 0
        
        self.frame = Frame(master, width=555, height=546)
        self.frame.pack(fill=BOTH, expand=YES)
        self.master = master
        self.x, self.y, self.w, self.h = -1,-1,-1,-1

        # bind master to <Configure> in order to handle any resizing, etc.
        # postpone self.master.bind("<Configure>", self.Master_Configure)
        self.master.bind('<Enter>', self.bindConfigure)
        #self.master.bind('<Button-1>', self.bindConfigure)
        master.bind('<Escape>', self.Clear_Pending_Actions)
        
        
        self.master.title("DigiPlot")
        
        # ======================  Make a Widget Frame =============
        
        # ------------ left_frame -----------------
        left_frame = Frame(self.frame)

        #self.Defined_Points_Label = Label(left_frame,text="Defined Points", width="15")
        #self.Defined_Points_Label.pack(anchor=NW, side=TOP)
        
        left_frame_ch1 = Frame(left_frame)
        
        self.Del_Button = Button(left_frame_ch1,text="Delete Point", width="10")
        self.Del_Button.pack(side=LEFT, anchor=NW)
        self.Del_Button.bind("<ButtonRelease-1>", self.Del_Button_Click)
        
        self.Sort_Button = Button(left_frame_ch1,text="Sort By X", width="9")
        self.Sort_Button.pack(side=LEFT, anchor=NW)
        self.Sort_Button.bind("<ButtonRelease-1>", self.Sort_Button_Click)

        
        self.Delete_All_Button = Button(left_frame_ch1,text="Delete All", width="10")
        self.Delete_All_Button.pack(side=LEFT, anchor=NW)
        self.Delete_All_Button.bind("<ButtonRelease-1>", self.Delete_All_Button_Click)
        
        left_frame_ch1.pack(fill=Y,  side=TOP, anchor=NW)


        # Listbox for points
        lbframe = Frame( left_frame )
        scrollbar = Scrollbar(lbframe, orient=VERTICAL)
        self.Defined_Points_Listbox = Listbox(lbframe, width="40", selectmode="single", 
                                              yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.Defined_Points_Listbox.yview)
        scrollbar.pack(side=RIGHT, fill=Y, expand=YES)
        self.Defined_Points_Listbox.pack(side=LEFT, fill=Y, expand=YES)
        lbframe.pack(fill=Y, anchor=W, side=TOP, expand=YES)
        self.Defined_Points_Listbox_frame = lbframe
        self.Defined_Points_Listbox.bind("<ButtonRelease-1>", self.Defined_Points_Listbox_Click)
        
        left_frame.pack(fill=Y,  side=LEFT, anchor=NW)

        # -------------------- right_frame -------------------
        # Canvas for plot
        right_frame = Frame(self.frame)
        right_frame.pack(fill=BOTH, expand=YES, side=LEFT, anchor=NE)
        
        right_frame_ch1 = Frame(right_frame)
        right_frame_ch1.pack(fill=Y,  side=TOP, anchor=NW)

        self.GreyScale_Checkbutton = Checkbutton(right_frame_ch1,text="Grey Scale Plot", width="15")
        self.GreyScale_Checkbutton.pack(side=LEFT, anchor=W)
        self.GreyScale_Checkbutton_StringVar = StringVar()
        self.GreyScale_Checkbutton_StringVar.set('yes')        
        self.GreyScale_Checkbutton.configure(variable=self.GreyScale_Checkbutton_StringVar, onvalue="yes", offvalue="no")
        self.GreyScale_Checkbutton_StringVar_traceName = self.GreyScale_Checkbutton_StringVar.trace_variable("w", self.GreyScale_Checkbutton_StringVar_Callback)

        self.LogX_Checkbutton = Checkbutton(right_frame_ch1,text="Log X Scale", width="15")
        self.LogX_Checkbutton.pack(side=LEFT, anchor=W)
        self.LogX_Checkbutton_StringVar = StringVar()
        self.LogX_Checkbutton_StringVar.set('no')        
        self.LogX_Checkbutton.configure(variable=self.LogX_Checkbutton_StringVar, onvalue="yes", offvalue="no")
        self.LogX_Checkbutton_StringVar_traceName = self.LogX_Checkbutton_StringVar.trace_variable("w", self.LogX_Checkbutton_StringVar_Callback)

        self.LogY_Checkbutton = Checkbutton(right_frame_ch1,text="Log Y Scale", width="15")
        self.LogY_Checkbutton.pack(side=LEFT, anchor=W)
        self.LogY_Checkbutton_StringVar = StringVar()
        self.LogY_Checkbutton_StringVar.set('no')        
        self.LogY_Checkbutton.configure(variable=self.LogY_Checkbutton_StringVar, onvalue="yes", offvalue="no")
        self.LogY_Checkbutton_StringVar_traceName = self.LogY_Checkbutton_StringVar.trace_variable("w", self.LogY_Checkbutton_StringVar_Callback)

        
        self.MakeBigger_Button = Button(right_frame_ch1,text="+", width="5")
        self.MakeBigger_Button.pack(side=LEFT)
        self.MakeBigger_Button.bind("<ButtonRelease-1>", self.MakeBigger_Button_Click)

        self.MakeSmaller_Button = Button(right_frame_ch1,text="-", width="5")
        self.MakeSmaller_Button.pack(side=LEFT)
        self.MakeSmaller_Button.bind("<ButtonRelease-1>", self.MakeSmaller_Button_Click)

        self.MakeFit_Button = Button(right_frame_ch1,text="Fit", width="5")
        self.MakeFit_Button.pack(side=LEFT)
        self.MakeFit_Button.bind("<ButtonRelease-1>", self.MakeFit_Button_Click)

        self.Plot_Canvas = Canvas(right_frame,width="600", height="600")
        self.Plot_Canvas.pack(fill=BOTH, expand=YES, side=TOP)
        self.Plot_Canvas.bind("<ButtonRelease-1>", self.Plot_Canvas_Click)
        self.Plot_Canvas.bind("<Configure>", self.Plot_Canvas_Resize)

        
        # Mouse Wheel for Zooming in and out
        self.master.bind("<MouseWheel>",self.MouseWheelHandler) # Windows Binding
        self.master.bind("<Button-4>",self.MouseWheelHandler) # Linux Binding
        self.master.bind("<Button-5>",self.MouseWheelHandler) # Linux Binding        
        self.Plot_Canvas.bind("<Button-3>", self.Canvas_Begin_End_Drag)
        self.Plot_Canvas.bind("<B3-Motion>", self.Canvas_Drag_Axes)
        self.Plot_Canvas.bind("<Motion>", self.Canvas_Hover)
        self.Plot_Canvas.bind("<Enter>", self.Canvas_Enter)
        self.Plot_Canvas.bind("<Leave>", self.Canvas_Leave)
        
        if HAS_IMAGEGRAB:
            self.Plot_Canvas.bind('<Control-v>', self.menu_Paste_Clipboard_Image_event)

        # ======================  End of Widget Frame =============
        # make a Status Bar
        self.statusMessage = StringVar()
        self.statusMessage.set("")
        self.statusbar = Label(self.master, textvariable=self.statusMessage, bd=1, relief=SUNKEN)
        self.statusbar.pack(anchor=SW, fill=X, side=BOTTOM)
        
        self.statusMessage.set("Welcome to temp")
        
        self.menuBar = Menu(self.master, relief = "raised", bd=2)

        top_File = Menu(self.menuBar, tearoff=0)

        top_File.add("command", label = "Read Image File", command = self.menu_File_Import_Image)
        if HAS_IMAGEGRAB:
            top_File.add("command", label = "Paste Clipboard Image", command = self.menu_Paste_Clipboard_Image)
        top_File.add("command", label = "Save Points to CSV", command = self.menu_File_Save_CSV)
        self.menuBar.add("cascade", label="File", menu=top_File)


        top_Clipboard = Menu(self.menuBar, tearoff=0)
        top_Clipboard.add("command", label = "Comma Separated (CSV)", command = self.menu_Clipboard_Comma)
        top_Clipboard.add("command", label = "Tab Separated (Excel)", command = self.menu_Clipboard_Tab)
        self.menuBar.add("cascade", label="Data-To-Clipboard", menu=top_Clipboard)

        top_Anchor_Plot = Menu(self.menuBar, tearoff=0)
        top_Anchor_Plot.add("command", label = "Set Xmin", command = self.menu_Anchor_Plot_Set_Xmin)
        top_Anchor_Plot.add("command", label = "Set Xmax", command = self.menu_Anchor_Plot_Set_Xmax)
        top_Anchor_Plot.add("command", label = "Set Ymin", command = self.menu_Anchor_Plot_Set_Ymin)
        top_Anchor_Plot.add("command", label = "Set Ymax", command = self.menu_Anchor_Plot_Set_Ymax)
        top_Anchor_Plot.add("command", label = "Set All", command = self.menu_Anchor_Plot_Set_All)
        self.menuBar.add("cascade", label="Anchor-Plot", menu=top_Anchor_Plot)


        top_Fix = Menu(self.menuBar, tearoff=0)
        top_Fix.add("command", label = "4 Point Axes Align", command = self.menu_Fix_Image_4Pt)
        top_Fix.add("command", label = "3 Point Axes Align", command = self.menu_Fix_Image_3Pt)
        self.menuBar.add("cascade", label="Fix-Image-Alignment", menu=top_Fix)


        top_View = Menu(self.menuBar, tearoff=0)
        top_View.add("command", label = "Fit Image", command = self.menu_Fit_Image)
        self.menuBar.add("cascade", label="View", menu=top_View)

        
        self.menuBar.add("command", label = "Help", command = self.menu_Help)

        top_Exit = Menu(self.menuBar, tearoff=0)
        top_Exit.add("command", label = "Quit", command = self.menu_Exit_Quit)
        self.menuBar.add("cascade", label="Exit", menu=top_Exit)

        self.master.config(menu=self.menuBar)
        
        self.master.sizefrom(who='program')
        self.master.resizable(True, True) # Linux may not respect this

        self.Plot_Canvas.focus_set()
            
        self.PA = PlotArea(w_canv=600, h_canv=600) # will hold PlotArea() when it is opened
        self.pointL = []
        self.has_some_new_data = False
        
        self.is_dragging = False
        self.last_right_click_pos = (0,0) # any action calling Canvas_Find_Closest will set

        self.last_hover_pos = (0,0)
        self.in_canvas = False

        self.canvas_tooltip_num = 0
        self.canvas_tooltip_inc = 1 # set > 1 to skip some options
        self.canvas_tooltip_strL = ['Left Click on X minimum.', 'Left Click on X maximum',
                                    'Left Click on Y minimum.', 'Left Click on Y maximum']
                                    
        # Units NEED to be fraction of img
        self.canvas_click_posL = [None, None, None, None] # only useful if all None's replaced
        self.distortion_flag = False

        master.protocol('WM_DELETE_WINDOW', self.cleanupOnQuit)
        
        # if image file given, then import it.
        if len( sys.argv ) == 2:
            fName = sys.argv[1]
            flo = fName.lower()
            if flo.endswith('.png') or flo.endswith('.jpg') or flo.endswith('.jpeg') or flo.endswith('.gif'):
                if os.path.isfile( fName ): # if file exists, read it as a definition file
                    if self.PA.open_img_file( fName ):
                        self.Initialize_Image_State()
                        self.statusMessage.set('file "%s" opened'%fName)
                        self.master.title("DigiPlot: %s"%fName)
                    else:
                        self.statusMessage.set('file "%s" failed to open'%fName)
                        
                else:
                    self.statusMessage.set('file "%s" not found'%fName)
        
        
        self.master.update_idletasks()
        # give a few milliseconds before calling bindConfigure
        self.master.after(100, lambda: self.bindConfigure(None))
            
    def Initialize_Image_State(self):
        
        # Clear entries in Listbox
        self.Defined_Points_Listbox.delete(0, END)
        self.LogX_Checkbutton_StringVar.set('no')        
        self.LogY_Checkbutton_StringVar.set('no')        
        
        self.pointL = []
        self.has_some_new_data = False
        
        self.is_dragging = False
        self.last_right_click_pos = (0,0) # any action calling Canvas_Find_Closest will set
        
        self.canvas_tooltip_num = 0
        self.canvas_tooltip_inc = 1 # set > 1 to skip some options
        
                                    
        # Units NEED to be fraction of img
        self.canvas_click_posL = [None, None, None, None] # only useful if all None's replaced
        self.distortion_flag = False
        
        self.PA.zoom_to_quadrant(qname='LL')
        self.plot_points()
        
    def Clear_Pending_Actions(self, event):
        self.statusMessage.set("Cleared pending actions.")
        self.is_dragging = False
        self.last_right_click_pos = (0,0) # any action calling Canvas_Find_Closest will set
        
        self.canvas_tooltip_num = 1000
        self.canvas_tooltip_inc = 1 
        
        self.plot_points()
        
        
    def cleanupOnQuit(self):
        
        if (len(self.pointL)>0) and self.has_some_new_data:
            if self.AskYesNo( title='Exit WITHOUT Saving Data?', \
                              message='Exit WITHOUT Saving Data??\n'+\
                              '(Hit "Yes" to DISCARD your data.)'):
                try:
                    self.master.destroy()
                except:
                    pass
                sys.exit(1)
        else:
            try:
                self.master.destroy()
            except:
                pass
            sys.exit(1)

    def bindConfigure(self, event):
        if not self.initComplete:
            self.initComplete = 1
            #self.Plot_Canvas.config(width=600, height=500)
            self.master.bind("<Configure>", self.Master_Configure)
            
            self.PA.zoom_to_quadrant(qname='LL')
            self.plot_points()


    def Master_Configure(self, event):
        
        if event.widget != self.master:
            if self.w > -1:
                return
        self.master.unbind('<Enter>')
            
        x = int(self.master.winfo_x())
        y = int(self.master.winfo_y())
        w = int(self.master.winfo_width())
        h = int(self.master.winfo_height())
        
        if (self.x, self.y, self.w, self.h) == (-1,-1,-1,-1):
            self.x, self.y, self.w, self.h = x,y,w,h
            self.lb_init_width  = int(self.Defined_Points_Listbox_frame.winfo_width() )
            self.lb_init_height = int(self.Defined_Points_Listbox_frame.winfo_height())
            self.w_init = w
            self.h_init = h
            self.w_canv_init = int( self.Plot_Canvas.winfo_width() )

        if self.w!=w or self.h!=h:
            #print( "Master reconfigured... make resize adjustments")
            self.w=w
            self.h=h
        
        #print('self.frame.w =', self.frame.winfo_width())
        self.frame.update()
        #try:
        #    self.frame.update_idletasks()
        #except:
        #    pass
        
        #dw = max(100, self.w - self.w_init + self.w_canv_init - 1)
        #dh = max(100, self.h - self.h_init + self.lb_init_height)
        
        #self.Plot_Canvas.config(width=dw, height=dh)
        
        #self.PA.set_canvas_wh(dw, dh)
        #self.plot_points()

    def Plot_Canvas_Resize(self, event):
        self.PA.set_canvas_wh(event.width, event.height)
        self.plot_points()
        
    def eval_distortion(self):
        if None in self.canvas_click_posL:
            self.distortion_flag = False
            return
        
        # evaluate any distortion in img 
        pos_xmin, pos_xmax, pos_ymin, pos_ymax = self.canvas_click_posL
        
        xy_xmin = self.PA.get_xy_at_fifj( *pos_xmin )
        xy_xmax = self.PA.get_xy_at_fifj( *pos_xmax )
        xy_ymin = self.PA.get_xy_at_fifj( *pos_ymin )
        xy_ymax = self.PA.get_xy_at_fifj( *pos_ymax )
        
        #for pos, xy in zip([pos_xmin, pos_xmax, pos_ymin, pos_ymax], [xy_xmin, xy_xmax, xy_ymin, xy_ymax]):
        #    print( 'pos=(%g,%g)'%pos,'  xy=(%g,%g)'%xy )
        
        x_err = xy_ymax[0] - xy_ymin[0]
        y_err = xy_xmax[1] - xy_xmin[1]
        
        pcent_x_err = abs(x_err) * 100.0 / max( abs(self.PA.x_origin), abs(self.PA.xmax) )
        pcent_y_err = abs(y_err) * 100.0 / max( abs(self.PA.y_origin), abs(self.PA.ymax) )
        
        self.statusMessage.set("Distortion Error: Xerr=%g%%, Yerr=%g%%"%(pcent_x_err, pcent_y_err))
        #self.ShowWarning( title='Distortion Error', message="Xerr=%g%%, Yerr=%g%%"%(pcent_x_err, pcent_y_err))
        
        
    def plot_points(self):
        s = 3
        s2 = s+3
        s3 = s+5
        
        # make coordinate axes
        self.Plot_Canvas.delete("all")
        
        # Place click directions onto canvas
        if self.canvas_tooltip_num < len(self.canvas_tooltip_strL):
            tt_str = self.canvas_tooltip_strL[ self.canvas_tooltip_num ]
        else:
            tt_str = ''
        
        greyscale = str(self.GreyScale_Checkbutton_StringVar.get())=='yes'
        self.photo_image = self.PA.get_tk_photoimage(greyscale=greyscale, text=tt_str, 
                                                     show_linlog_text=True)
        self.Plot_Canvas.create_image(0,0, anchor=NW, image=self.photo_image )
                    
        iL = [i for i in self.Defined_Points_Listbox.curselection()]
        if (len(iL)>0):
            isel = iL[0]
        else:
            isel = -1
        
        # show x and y axes if they are visible
        if (self.canvas_tooltip_num>0) and self.PA.x_is_visible( self.PA.x_origin ):
            i = self.PA.get_canvas_i( self.PA.x_origin )
            j1, j2 = 0, self.PA.h_canv
            self.Plot_Canvas.create_line(i, j1, i, j2, fill="purple", dash=(1,5,1,1), width=3)
        if (self.canvas_tooltip_num>2) and self.PA.y_is_visible( self.PA.y_origin ):
            j = self.PA.get_canvas_j( self.PA.y_origin )
            i1, i2 = 0, self.PA.w_canv
            self.Plot_Canvas.create_line(i1, j, i2, j, fill="purple", dash=(1,5,1,1), width=3)
            
        # Show max click points if available
        pos_xmin, pos_xmax, pos_ymin, pos_ymax = self.canvas_click_posL
        if pos_xmax is not None:
            i = self.PA.get_canvas_i_from_img_fi( pos_xmax[0] )
            j = self.PA.get_canvas_j_from_img_fj( pos_xmax[1] )
            self.Plot_Canvas.create_line(i, j-s2, i, j+s2, fill="purple", width=3)
            self.Plot_Canvas.create_line(i-s2, j, i+s2, j, fill="purple", width=3)

        if pos_ymax is not None:
            i = self.PA.get_canvas_i_from_img_fi( pos_ymax[0] )
            j = self.PA.get_canvas_j_from_img_fj( pos_ymax[1] )
            self.Plot_Canvas.create_line(i, j-s2, i, j+s2, fill="purple", width=3)
            self.Plot_Canvas.create_line(i-s2, j, i+s2, j, fill="purple", width=3)

        # Show points
        for ip,P in enumerate(self.pointL):
            x,y = P.get_xy()
            i,j = self.PA.get_ij_at_xy( x,y )
            
            if min(i,j)>=0:
                if (isel>=0) and (ip==isel):
                    self.Plot_Canvas.create_rectangle((i-s2, j-s2, i+s2, j+s2), outline="blue", fill="blue")
                else:
                    self.Plot_Canvas.create_rectangle((i-s2, j-s2, i+s2, j+s2), outline="blue", fill="cyan")

        # Show cross-hairs for picking axis points
        if self.in_canvas:
            if self.canvas_tooltip_num < len(self.canvas_tooltip_strL):
                x,y = self.last_hover_pos
                self.Plot_Canvas.create_line(x, 0, x, self.PA.h_canv, fill="magenta", width=3, dash=(5,) )
                self.Plot_Canvas.create_line(0, y, self.PA.w_canv, y, fill="magenta", width=3, dash=(5,) )
            else:
                x,y = self.last_hover_pos
                self.Plot_Canvas.create_line(x, 0, x, self.PA.h_canv, fill="cyan", width=3, dash=(5,) )
                self.Plot_Canvas.create_line(0, y, self.PA.w_canv, y, fill="cyan", width=3, dash=(5,) )
        
    
    def add_point(self, x_float, y_float):
        
        self.has_some_new_data = True
        
        itemL = list(self.Defined_Points_Listbox.get(0, END))
        iL = [i for i in self.Defined_Points_Listbox.curselection()]
        
        if len(itemL)==0:
            self.pointL.append( Point(x_float, y_float) )
            self.Defined_Points_Listbox.insert(END, self.pointL[-1].get_str() )
            self.select_pointL(0)
        else:
            
            if len(iL)==0:
                self.pointL.append( Point(x_float, y_float) )
                self.Defined_Points_Listbox.insert(END, self.pointL[-1].get_str() )
                self.select_pointL(END)
            else:
                self.pointL.insert(iL[0]+1, Point(x_float, y_float) )
                self.Defined_Points_Listbox.insert(iL[0]+1, self.pointL[iL[0]+1].get_str() )
                self.select_pointL(iL[0]+1)
                
                #print( 'self.pointL =',self.pointL)
                
        self.statusMessage.set("added point x,y=%g, %g"%(x_float, y_float))

    def Canvas_Enter(self, event):
        self.in_canvas = True
        self.plot_points()
    def Canvas_Leave(self, event):
        self.in_canvas = False
        self.plot_points()


    def Canvas_Hover(self, event):
        
        if self.canvas_tooltip_num >= len(self.canvas_tooltip_strL):
        
            x = int(event.x)
            y = int(event.y)
            
            x_move, y_move = self.PA.get_xy_at_ij(x,y)
            
            try:
                x_str = '%g'%x_move
                y_str = '%g'%y_move
            except:
                x_str = '%s'%x_move
                y_str = '%s'%y_move
            
            self.statusMessage.set(x_str + ', ' + y_str)
            
        self.last_hover_pos = (event.x, event.y)
        self.plot_points()
            

    def Canvas_Begin_End_Drag(self, event):
        
        self.is_dragging = not self.is_dragging
        ix = int(event.x)
        iy = int(event.y)
        
        self.last_right_click_pos = (ix, iy)
        
        if self.is_dragging:
            self.statusMessage.set("Dragging Selected Point")
        else:
            self.statusMessage.set("END Dragging Selected Point")
        

    def Canvas_Drag_Axes(self, event):
        
        di = self.last_right_click_pos[0] - event.x
        dj = self.last_right_click_pos[1] - event.y
        self.PA.adjust_offset(di, dj)
        
        self.last_right_click_pos = (event.x, event.y)
        
        self.plot_points()

    def MouseWheelHandler(self, event):

        if event.num == 5 or event.delta < 0:
            result = -1 
            #self.PA.zoom_in(zoom_factor=0.1)
            self.PA.zoom_into_ij(event.x, event.y, zoom_factor=0.1)
        else:
            result = 1 
            #self.PA.zoom_out(zoom_factor=0.1)
            self.PA.zoom_out_from_ij(event.x, event.y, zoom_factor=0.1)
            
        self.plot_points()
    
    def Sort_Button_Click(self, event):
        self.statusMessage.set("Sorting Points by Increasing X")
        
        self.pointL.sort( key=lambda P: (P.x,P.y) )
        
        # Clear entries in Listbox
        self.Defined_Points_Listbox.delete(0, END)
        
        for P in self.pointL:
            self.Defined_Points_Listbox.insert(END, P.get_str() )
            #print(P.get_str())
        
        self.Defined_Points_Listbox.select_set( END )
        self.plot_points()
    
    def Delete_All_Button_Click(self, event):
        if self.AskOK_Cancel( title='Delete All Data Points?', 
                              message='Are you sure you want to delete ALL data points?'):
                                  
            self.Defined_Points_Listbox.delete(0, END)            
            self.pointL = []
            self.has_some_new_data = False
            self.plot_points()
            
            self.statusMessage.set("Deleted All Data Points.")
        else:
            self.statusMessage.set("Cancelled Delete.")
    
    def Del_Button_Click(self, event):
        
        iL = [i for i in self.Defined_Points_Listbox.curselection()]
        if len(iL)>0:
            #print 'Deleting item %i'%iL[0]
            self.statusMessage.set("Deleting Pt: %s"%self.Defined_Points_Listbox.get( iL[0] ))
            
            self.Defined_Points_Listbox.delete( iL[0] )
            self.pointL.pop( iL[0] )
            
            isel = iL[0]-1
            if isel < 0:
                isel = 0
            if len(self.pointL)>0:
                self.select_pointL(isel)
            
        self.plot_points()


    def MakeBigger_Button_Click(self, event):
        self.PA.zoom_in(zoom_factor=0.1)
        self.plot_points()
        
    def MakeSmaller_Button_Click(self, event):
        self.PA.zoom_out(zoom_factor=0.1)
        self.plot_points()
        
    def MakeFit_Button_Click(self, event):
        self.menu_Fit_Image()
        self.plot_points()


    def select_pointL(self, i ):
        
        self.Defined_Points_Listbox.selection_clear(0, END)
        if i==END:
            self.Defined_Points_Listbox.select_set( END )
            i = len(self.pointL)-1
        else:
            self.Defined_Points_Listbox.select_set( i )
            
        self.plot_points()

    def menu_Paste_Clipboard_Image_event(self, event):
        self.menu_Paste_Clipboard_Image()

    def menu_Paste_Clipboard_Image(self):
        self.statusMessage.set("Pasting Image from Clipboard")
        
        if self.PA.set_img_from_clipboard():
            self.master.title("DigiPlot: Pasted Clipboard Image")
            self.Initialize_Image_State()
            
            self.statusMessage.set("SUCCESS... Pasted Image from Clipboard")
        else:
            self.statusMessage.set("FAILED... Pasting Image from Clipboard")
            self.ShowWarning(title='Clipboard Image Paste Failed', message='Clipboard Image Paste Failed.')
        

    def menu_File_Import_Image(self):
        self.statusMessage.set("called menu_File_Import_Image")
        
        if (len(self.pointL)>0) and self.has_some_new_data:
            if not self.AskYesNo( title='Current Data NOT Saved.', \
                              message='Import New Image?\n'+\
                              '(Hit "Yes" to DISCARD your current data.)'):
                return
        
        
        filetypes = [
            ('Images','*.png;*.jpg;*.gif;*.jpeg'),
            ('Any File','*.*')]
        img_path = tkFileDialog.askopenfilename(parent=self.master,title='Open Image file', 
            filetypes=filetypes)
            
        if img_path:
            if self.PA.open_img_file( img_path ):
                head,fName = os.path.split( img_path )
                self.master.title("DigiPlot: %s"%fName)
                self.Initialize_Image_State()
                self.statusMessage.set('file "%s" opened'%fName)
            


    def menu_File_Save_CSV(self):
        self.statusMessage.set("called menu_File_Save_CSV")
        self.saveFile('*.csv')
        
        
    def saveFile(self, fileDesc='*.csv'):
        
        fsave = tkFileDialog.asksaveasfilename(parent=self.master, title='Saving CSV file', 
            initialfile=fileDesc)
        
        if fsave:
            if fsave.find('.')<0:
                fsave += '.csv'
            
            
            self.statusMessage.set( 'Saving to:' + fsave )
            
            self.master.title("DigiPlot: %s"%fsave)
            
            fOut = open( fsave, 'w' )
            
            for P in self.pointL:
                fOut.write( '%s\n'%P.get_str() )
            fOut.close()
            
            self.has_some_new_data = False # after successful save
    
    def menu_Clipboard_Comma(self):
        if len(self.pointL)==0:
            self.ShowWarning( title='No Data Warning', message='No Data to put on Clipboard.')
            return
        
        self.frame.clipboard_clear()
        for P in self.pointL:
            self.frame.clipboard_append( P.get_str() + '\n' )
        self.statusMessage.set( "====== Comma-Separated on Clipboard =========")
        
    def menu_Clipboard_Tab(self):
        if len(self.pointL)==0:
            self.ShowWarning( title='No Data Warning', message='No Data to put on Clipboard.')
            return
        
        self.frame.clipboard_clear()
        for P in self.pointL:
            self.frame.clipboard_append( P.get_str().replace(', ','\t') + '\n' )
        self.statusMessage.set( "====== Tab-Separated on Clipboard =========")

    def set_xy_min_max(self, tt_num):
        
        if tt_num==0:
            self.PA.zoom_to_quadrant(qname='LL')
        elif tt_num==1:
            self.PA.zoom_to_quadrant(qname='LR')
        elif tt_num==2:
            self.PA.zoom_to_quadrant(qname='LL')
        elif tt_num==3:
            self.PA.zoom_to_quadrant(qname='UL')
        
        self.canvas_tooltip_num = tt_num
        self.canvas_tooltip_inc = 10
        self.plot_points()

    def menu_Anchor_Plot_Set_Xmin(self):
        self.statusMessage.set("Set Axis Point Xmin")
        self.set_xy_min_max(0)


    def menu_Anchor_Plot_Set_Xmax(self):
        self.statusMessage.set("Set Axis Point Xmax")
        self.set_xy_min_max(1)


    def menu_Anchor_Plot_Set_Ymin(self):
        self.statusMessage.set("Set Axis Point Ymin")
        self.set_xy_min_max(2)


    def menu_Anchor_Plot_Set_Ymax(self):
        self.statusMessage.set("Set Axis Point Ymax")
        self.set_xy_min_max(3)

    def menu_Anchor_Plot_Set_All(self):
        self.statusMessage.set("Set All 4 Axis Points (Xmin, Xmax, Ymin, Ymax)")
        self.set_xy_min_max(0)
        self.canvas_tooltip_inc = 1

    def menu_Fit_Image(self):
        self.PA.fit_img_on_canvas()
        self.plot_points()

    def menu_Fix_Image_4Pt(self):
        dialog = _ReAlign(self.master, "4 Point Re-Align Plot Axes (Make Orthogonal)",\
                          dialogOptions={'img':self.PA.img})
        if dialog.result is not None:
            self.PA.set_img( dialog.result["img_fixed"] )
            
            self.master.title("DigiPlot: Re-Aligned Image")
            self.Initialize_Image_State()
            self.statusMessage.set('Re-Aligned Image')
            
        dialog.destroy()

    def menu_Fix_Image_3Pt(self):
        dialog = _ReAlign(self.master, "3 Point Re-Align Plot Axes (Make Orthogonal)",\
                          dialogOptions={'img':self.PA.img, 'use_3_point':True})
        if dialog.result is not None:
            self.PA.set_img( dialog.result["img_fixed"] )
            
            self.master.title("DigiPlot: Re-Aligned Image")
            self.Initialize_Image_State()
            self.statusMessage.set('Re-Aligned Image')
            
        dialog.destroy()

    def menu_Help(self):
        self.statusMessage.set("called menu_Help")
        help_msg = """
        
Left Click = Add Point or Define Axes
Right Drag = Pan Zoomed Plot
Mouse Wheel = Zoom

Esc Key = Clear Pending Actions
"""
        self.ShowInfo( title='Help', message='Mouse/Keyboard Options\r' + help_msg)


    def menu_Exit_Quit(self):
        self.statusMessage.set("called menu_Exit_Quit")
        print( "called menu_Exit_Quit")
        self.cleanupOnQuit()

    def Plot_Canvas_Click(self, event): #click method for component ID=1

        x,y = self.PA.get_xy_at_ij(event.x, event.y)

        if self.canvas_tooltip_num < len(self.canvas_tooltip_strL):
            tt_str = self.canvas_tooltip_strL[ self.canvas_tooltip_num ]
            
            self.canvas_click_posL[self.canvas_tooltip_num] = self.PA.get_fifj_at_ij(event.x, event.y) # can detect img distortion
            
            if self.canvas_tooltip_num==0:
                result = tkSimpleDialog.askfloat(tt_str, 'Enter Xmin')
                if result is not None:
                    self.statusMessage.set("Xmin = %g"%result)
                    self.PA.set_ix_origin(event.x, result)
                    self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==1:
                result = tkSimpleDialog.askfloat(tt_str, 'Enter Xmax')
                if result is not None:
                    self.statusMessage.set("Xmax = %g"%result)
                    self.PA.set_imax_xmax(event.x, result)
                    self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==2:
                result = tkSimpleDialog.askfloat(tt_str, 'Enter Ymin')
                if result is not None:
                    self.statusMessage.set("Ymin = %g"%result)
                    self.PA.set_jy_origin(event.y, result)
                    self.canvas_tooltip_num += self.canvas_tooltip_inc
                
            elif self.canvas_tooltip_num==3:
                result = tkSimpleDialog.askfloat(tt_str, 'Enter Ymax')
                if result is not None:
                    self.statusMessage.set("Ymax = %g"%result)
                    self.PA.set_jmax_ymax(event.y, result)
                    self.canvas_tooltip_num += self.canvas_tooltip_inc
                    
                    self.PA.fit_img_on_canvas()
            
            self.canvas_tooltip_inc = 1 # Just in case it is >1 for single options

        
            if self.canvas_tooltip_num==0:
                self.PA.zoom_to_quadrant(qname='LL')
            elif self.canvas_tooltip_num==1:
                self.PA.zoom_to_quadrant(qname='LR')
            elif self.canvas_tooltip_num==2:
                self.PA.zoom_to_quadrant(qname='LL')
            elif self.canvas_tooltip_num==3:
                self.PA.zoom_to_quadrant(qname='UL')


            if not None in self.canvas_click_posL:
                self.eval_distortion()
            
            self.plot_points()
                
        else:
        
            #print( "executed method Plot_Canvas_Click")
            #self.statusMessage.set("executed method Plot_Canvas_Click")
            #print( "clicked in canvas at i,j =",event.x,event.y)
            
            #print('x=%g, y=%g'%(x,y))
            self.add_point(x,y)

    def Defined_Points_Listbox_Click(self, event): #click method for component ID=2
        #print( "current selection(s) =",self.Defined_Points_Listbox.curselection())
        labelL = []
        for i in self.Defined_Points_Listbox.curselection():
            labelL.append( self.Defined_Points_Listbox.get(i))
        
        if labelL:
            self.statusMessage.set("Selected Pt: %s"%labelL[0])
        self.plot_points()

    def GreyScale_Checkbutton_StringVar_Callback(self, varName, index, mode):
        self.statusMessage.set("    GreyScale_Checkbutton_StringVar = "+self.GreyScale_Checkbutton_StringVar.get())
        #print "    new StringVar value =",self.GreyScale_Checkbutton_StringVar.get()
        self.plot_points()

    def LogX_Checkbutton_StringVar_Callback(self, varName, index, mode):
        self.statusMessage.set("    LogX_Checkbutton_StringVar = "+self.LogX_Checkbutton_StringVar.get())
        
        is_log = str(self.LogX_Checkbutton_StringVar.get())=='yes'
        if is_log:
            self.PA.set_log_x()
        else:
            self.PA.set_linear_x()
            
        self.plot_points()

    def LogY_Checkbutton_StringVar_Callback(self, varName, index, mode):
        self.statusMessage.set("    LogY_Checkbutton_StringVar = "+self.LogY_Checkbutton_StringVar.get())
        
        is_log = str(self.LogY_Checkbutton_StringVar.get())=='yes'
        if is_log:
            self.PA.set_log_y()
        else:
            self.PA.set_linear_y()
            
        self.plot_points()


    # standard message dialogs... showinfo, showwarning, showerror
    def ShowInfo(self, title='Title', message='your message here.'):
        tkMessageBox.showinfo( title, message )
        return
    def ShowWarning(self, title='Title', message='your message here.'):
        tkMessageBox.showwarning( title, message )
        return
    def ShowError(self, title='Title', message='your message here.'):
        tkMessageBox.showerror( title, message )
        return
        
    # standard question dialogs... askquestion, askokcancel, askyesno, or askretrycancel
    # return True for OK, Yes, Retry, False for Cancel or No
    def AskYesNo(self, title='Title', message='your question here.'):
        return tkMessageBox.askyesno( title, message )
    def AskOK_Cancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askokcancel( title, message )
    def AskRetryCancel(self, title='Title', message='your question here.'):
        return tkMessageBox.askretrycancel( title, message )
        
    # return "yes" for Yes, "no" for No
    def AskQuestion(self, title='Title', message='your question here.'):
        return tkMessageBox.askquestion( title, message )
    # END of standard message dialogs

    # >>>>>>insert any user code below this comment for section "standard_message_dialogs"


    # standard file dialogs... askdirectory, askopenfile, asksaveasfilename

    # return a string containing directory name
    def AskDirectory(self, title='Choose Directory', initialdir="."):
        dirname = tkFileDialog.askdirectory(parent=self.master,initialdir=initialdir,title=title)
        return dirname # <-- string
        
    # return an OPEN file type object OR None (opened using mode, 'r','rb','w','wb')
    # WARNING... opening file with mode 'w' or 'wb' will erase contents
    def AskOpenFile(self, title='Choose File', mode='rb', initialdir='.', filetypes=None):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]
        fileobj = tkFileDialog.askopenfile(parent=self.master,mode=mode,title=title,
            initialdir=initialdir, filetypes=filetypes)
        
        # if opened, then fileobj.name contains the name string
        return fileobj # <-- an opened file, or the value None
        
    # return a string containing file name (the calling routine will need to open the file)
    def AskSaveasFilename(self, title='Save File', filetypes=None, initialfile=''):
        if filetypes==None:
            filetypes = [
                ('Text File','*.txt'),
                ('Data File','*.dat'),
                ('Output File','*.out'),
                ('Any File','*.*')]

        fileName = tkFileDialog.asksaveasfilename(parent=self.master,filetypes=filetypes, initialfile=initialfile ,title=title)
        return fileName # <-- string
        
    # END of standard file dialogs

    # >>>>>>insert any user code below this comment for section "standard_file_dialogs"



def main():
    root = Tk()
    app = DigiPlot(root)
    root.mainloop()

if __name__ == '__main__':
    main()



"""
This example demonstrates the proper use of project: digiplot
"""
import sys
import os

sys.path.insert(0, os.path.abspath("../../"))  # needed to find digiplot development version

from digiplot.digiplot import DigiPlot

# TODO: create working example of project digiplot

my_class = DigiPlot()




import numpy
# From: https://stackoverflow.com/questions/14177744/how-does-perspective-transformation-work-in-pil
from PIL import Image

def find_coeffs(pa, pb):
    """
    pa and pb contain 4 corresponding points on image in order UL,UR,LR,LL
    UL=(0,0), UR=(w,0), LR=(w,h), LL=(0,h)
    
    Units for pa and pb are pixels.
    pa is target plane (e.g. [(0, 0), (256, 0), (256, 256), (0, 256)])
    pb is source plane (e.g. [(0, 0), (256, 0), (new_width, height), (xshift, height)])
    """
    matrix = []
    for p1, p2 in zip(pa, pb):
        matrix.append([p1[0], p1[1], 1, 0, 0, 0, -p2[0]*p1[0], -p2[0]*p1[1]])
        matrix.append([0, 0, 0, p1[0], p1[1], 1, -p2[1]*p1[0], -p2[1]*p1[1]])

    A = numpy.matrix(matrix, dtype=numpy.float)
    B = numpy.array(pb).reshape(8)

    res = numpy.dot(numpy.linalg.inv(A.T * A) * A.T, B)
    # take commenters advice to avoid A.T
    #res = numpy.linalg.solve(A, B)
    return numpy.array(res).reshape(8)

def fix_plot_img( UL, UR, LR, LL, img):
    """
    Correct plot so that xmin and xmax have same y value.
    And, ymin/ymax have same x value
    """
    
    # source points
    pb = [UL,UR,LR,LL]
    
    xlo = (LL[0]+UL[0])/2
    xhi = (UR[0]+LR[0])/2
    
    yhi = (UL[1]+UL[1])/2
    ylo = (LL[1]+LR[1])/2
    
    # make target points
    UL = (xlo, yhi)
    UR = (xhi, yhi)
    LR = (xhi, ylo)
    LL = (xlo, ylo)
    pa = [UL,UR,LR,LL]

    # get most common color in original
    w,h = img.size
    img_rgba = img.convert('RGBA')
    pixels = img_rgba.getcolors(w*h)
    most_frequent_pixel = pixels[0][1]
    print( 'most_frequent_pixel =',most_frequent_pixel ) # (count, (color))  e.g. (505888, (255, 255, 255, 255))

    # fix original image (may have black corners from rotation/transform)
    coeffs = find_coeffs(pa, pb)
    img_fixed = img_rgba.transform(img.size, Image.PERSPECTIVE, coeffs, Image.BICUBIC)
    fff = Image.new('RGBA', img_rgba.size, most_frequent_pixel)
    img_out = Image.composite(img_fixed, fff, img_fixed)
    
    return img_out
    
if __name__=="__main__":
    
    img_name = "rot_poly_p3.jpg"
    UL = (105., 93.)
    UR = (848., 51.)
    LR = (877., 627.)
    LL = (135., 666.)
    img = Image.open(img_name)
    print('img.size =',img.size)
    
    img_und = fix_plot_img( UL, UR, LR, LL, img)
    img_und.save('undistort_'+img_name)
    
    
    
    





from pylab import *

xL = range(40)
yL = [2. + 0.2*x +0.02*x**2  for x in xL]

plot(xL, yL, label='y = 2 + 0.2x + 0.02x^2', linewidth=3)

legend(loc='upper left')
grid(True)
title( 'Digitizing Sample Plot\nVerify Results with Excel Trendline' )
xlabel( 'X Value' )
ylabel( 'Y Value' )
savefig('sample_poly.png', dpi=120)

show()



"""A setuptools based setup module for DigiPlot

See:
http://digiplot.readthedocs.org/en/latest/
https://github.com/sonofeft/DigiPlot

If installing from source, then
the best way to install DigiPlot is to use pip after navigating to the source directory::

    cd <path to where setup.py is located>
    pip install -e .

This will execute the setup.py file and insure that its pip-specific commands are run.

"""


# Always prefer setuptools over distutils
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# Place install_requires into the text file "requirements.txt"
with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f2:
    requires = f2.read().strip().splitlines()

target_file = path.join( here, 'digiplot','_version.py')
exec( open( target_file ).read() )  # creates local __version__ variable


setup(
    name='digiplot',
    version = __version__,  # METADATA_RESET:    version = '<<version>>',

    description = 'A tkinter code that helps users digitize curves from plot images.',
    long_description = long_description,

    # The project's main homepage.
    url='http://digiplot.readthedocs.org/en/latest/',
    download_url='https://github.com/sonofeft/DigiPlot',

    # Author details
    author = 'Charlie Taylor',
    author_email = 'cet@appliedpython.com',

    # license
    license = 'GPL-3',

    classifiers = [
        # Common status values are: "3 - Alpha", "4 - Beta", "5 - Production/Stable"
        'Development Status :: 4 - Beta',

        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        "Intended Audience :: End Users/Desktop",
        'Topic :: Software Development :: Build Tools',

        # This license should match "license" above
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    platforms = 'any',

    # What does your project relate to?
    keywords = 'digiplot setuptools development',

    packages = find_packages(exclude=['.tox', '.hg', 'docs']),
    package_data = {'digiplot':['examples/*.*', 'fonts/*.*']},

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed.
    install_requires = requires,  # read from requirements.txt

    tests_require = ['nose','coverage'], # ...OR... ['pytest','pytest-cov']
    test_suite='digiplot.tests', # allows "setup.py test" to work

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require = {
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    zip_safe= False,

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.

    entry_points = {
        'console_scripts': [
            'digiplot=digiplot.tk_digiplot:main',
        ],
    },
)